import React from 'react';

function isTrusy(obj) {
  return !!obj;
}

function isUndefined(obj) {
  return typeof obj === 'undefined';
}

var _extends = Object.assign || function (target) {
  for (var i = 1; i < arguments.length; i++) {
    var source = arguments[i];

    for (var key in source) {
      if (Object.prototype.hasOwnProperty.call(source, key)) {
        target[key] = source[key];
      }
    }
  }

  return target;
};

var objectWithoutProperties = function (obj, keys) {
  var target = {};

  for (var i in obj) {
    if (keys.indexOf(i) >= 0) continue;
    if (!Object.prototype.hasOwnProperty.call(obj, i)) continue;
    target[i] = obj[i];
  }

  return target;
};

function doIfElseOperation(props, Component) {
  var condition = props.$if,
      elseBlock = props.$else,
      restProps = objectWithoutProperties(props, ['$if', '$else']);


  if (isTrusy(condition)) {
    return React.createElement(Component, restProps);
  }

  if (typeof elseBlock === 'function') {
    return React.createElement(elseBlock, restProps);
  }
  return elseBlock || null;
}

function doSwitchCaseOperation(props, Component) {
  var switchValue = props.$switch,
      children = props.children,
      restProps = objectWithoutProperties(props, ['$switch', 'children']);


  var targetChildNode = null;

  var totalNoOfNode = React.Children.count(children);
  for (var i = 0; i < totalNoOfNode; i++) {
    var child = totalNoOfNode === 1 ? children : children[i];

    if (!React.isValidElement(child)) continue;

    var _child$props = child.props,
        caseValue = _child$props.$case,
        defaultValue = _child$props.$default;


    if (isUndefined(caseValue) && isUndefined(defaultValue)) {
      throw new Error('$case or $default is missing', child);
    }

    if (isTrusy(defaultValue) || caseValue === switchValue) {
      targetChildNode = child;
      break;
    }
  }

  if (targetChildNode) {
    var _targetChildNode$prop = targetChildNode.props,
        $case = _targetChildNode$prop.$case,
        $default = _targetChildNode$prop.$default,
        childProps = objectWithoutProperties(_targetChildNode$prop, ['$case', '$default']);

    targetChildNode = React.cloneElement(targetChildNode, childProps);
  }

  return React.createElement(Component, restProps, targetChildNode);
}

function doForLoopOperation(props, Component) {
  var items = props.$for,
      keyFn = props.$key,
      restProps = objectWithoutProperties(props, ['$for', '$key']);


  var children = items.map(function (item, index, array) {
    var child = props.children(item, index, array);
    var key = keyFn ? keyFn(item) : child.key;
    return React.cloneElement(child, _extends({ key: key }, child.props));
  });

  return React.createElement(Component, restProps, children);
}

var withTemplating = function withTemplating(Component) {
  var Templating = function Templating(props) {
    var $if = props.$if,
        $switch = props.$switch,
        $for = props.$for,
        restProps = objectWithoutProperties(props, ['$if', '$switch', '$for']);


    if (!isUndefined($if)) {
      return doIfElseOperation(props, Component);
    }

    if (!isUndefined($switch)) {
      return doSwitchCaseOperation(props, Component);
    }

    if (!isUndefined($for)) {
      return doForLoopOperation(props, Component);
    }

    return React.createElement(Component, restProps);
  };

  Templating.displayName = 'withTemplating(' + (Component.displayName || Component.name) + ')';
  return Templating;
};

// Thanks @sindresorhus
// https://raw.githubusercontent.com/sindresorhus/html-tags/master/html-tags.json

var HTML_TAGS = ['a', 'abbr', 'address', 'area', 'article', 'aside', 'audio', 'b', 'base', 'bdi', 'bdo', 'blockquote', 'body', 'br', 'button', 'canvas', 'caption', 'cite', 'code', 'col', 'colgroup', 'data', 'datalist', 'dd', 'del', 'details', 'dfn', 'dialog', 'div', 'dl', 'dt', 'em', 'embed', 'fieldset', 'figcaption', 'figure', 'footer', 'form', 'h1', 'h2', 'h3', 'h4', 'h5', 'h6', 'head', 'header', 'hgroup', 'hr', 'html', 'i', 'iframe', 'img', 'input', 'ins', 'kbd', 'keygen', 'label', 'legend', 'li', 'link', 'main', 'map', 'mark', 'math', 'menu', 'menuitem', 'meta', 'meter', 'nav', 'noscript', 'object', 'ol', 'optgroup', 'option', 'output', 'p', 'param', 'picture', 'pre', 'progress', 'q', 'rb', 'rp', 'rt', 'rtc', 'ruby', 's', 'samp', 'script', 'section', 'select', 'slot', 'small', 'source', 'span', 'strong', 'style', 'sub', 'summary', 'sup', 'svg', 'table', 'tbody', 'td', 'template', 'textarea', 'tfoot', 'th', 'thead', 'time', 'title', 'tr', 'track', 'u', 'ul', 'var', 'video', 'wbr'];

var DOMElements = {};

HTML_TAGS.forEach(function (tag) {
  var Fn = function Fn(props) {
    var $if = props.$if,
        $switch = props.$switch,
        $case = props.$case,
        $default = props.$default,
        $else = props.$else,
        $key = props.$key,
        $for = props.$for,
        rest = objectWithoutProperties(props, ['$if', '$switch', '$case', '$default', '$else', '$key', '$for']);

    return React.createElement(tag, rest);
  };
  var Tag = capitalCase(tag);
  Fn.displayName = Tag;
  DOMElements[Tag] = withTemplating(Fn);
});

function capitalCase(str) {
  return str.charAt(0).toUpperCase() + str.slice(1);
}

var Fragment = function Fragment(_ref) {
  var children = _ref.children;

  return children;
};

var Fragment$1 = withTemplating(Fragment);

var A = DOMElements.A,
    Abbr = DOMElements.Abbr,
    Address = DOMElements.Address,
    Area = DOMElements.Area,
    Article = DOMElements.Article,
    Aside = DOMElements.Aside,
    Audio = DOMElements.Audio,
    B = DOMElements.B,
    Base = DOMElements.Base,
    Bdi = DOMElements.Bdi,
    Bdo = DOMElements.Bdo,
    Blockquote = DOMElements.Blockquote,
    Body = DOMElements.Body,
    Br = DOMElements.Br,
    Button = DOMElements.Button,
    Canvas = DOMElements.Canvas,
    Caption = DOMElements.Caption,
    Cite = DOMElements.Cite,
    Code = DOMElements.Code,
    Col = DOMElements.Col,
    Colgroup = DOMElements.Colgroup,
    Data = DOMElements.Data,
    Datalist = DOMElements.Datalist,
    Dd = DOMElements.Dd,
    Del = DOMElements.Del,
    Details = DOMElements.Details,
    Dfn = DOMElements.Dfn,
    Dialog = DOMElements.Dialog,
    Div = DOMElements.Div,
    Dl = DOMElements.Dl,
    Dt = DOMElements.Dt,
    Em = DOMElements.Em,
    Embed = DOMElements.Embed,
    Fieldset = DOMElements.Fieldset,
    Figcaption = DOMElements.Figcaption,
    Figure = DOMElements.Figure,
    Footer = DOMElements.Footer,
    Form = DOMElements.Form,
    H1 = DOMElements.H1,
    H2 = DOMElements.H2,
    H3 = DOMElements.H3,
    H4 = DOMElements.H4,
    H5 = DOMElements.H5,
    H6 = DOMElements.H6,
    Head = DOMElements.Head,
    Header = DOMElements.Header,
    Hgroup = DOMElements.Hgroup,
    Hr = DOMElements.Hr,
    Html = DOMElements.Html,
    I = DOMElements.I,
    Iframe = DOMElements.Iframe,
    Img = DOMElements.Img,
    Input = DOMElements.Input,
    Ins = DOMElements.Ins,
    Kbd = DOMElements.Kbd,
    Keygen = DOMElements.Keygen,
    Label = DOMElements.Label,
    Legend = DOMElements.Legend,
    Li = DOMElements.Li,
    Link = DOMElements.Link,
    Main = DOMElements.Main,
    Map = DOMElements.Map,
    Mark = DOMElements.Mark,
    Math = DOMElements.Math,
    Menu = DOMElements.Menu,
    Menuitem = DOMElements.Menuitem,
    Meta = DOMElements.Meta,
    Meter = DOMElements.Meter,
    Nav = DOMElements.Nav,
    Noscript = DOMElements.Noscript,
    Object$1 = DOMElements.Object,
    Ol = DOMElements.Ol,
    Optgroup = DOMElements.Optgroup,
    Option = DOMElements.Option,
    Output = DOMElements.Output,
    P = DOMElements.P,
    Param = DOMElements.Param,
    Picture = DOMElements.Picture,
    Pre = DOMElements.Pre,
    Progress = DOMElements.Progress,
    Q = DOMElements.Q,
    Rb = DOMElements.Rb,
    Rp = DOMElements.Rp,
    Rt = DOMElements.Rt,
    Rtc = DOMElements.Rtc,
    Ruby = DOMElements.Ruby,
    S = DOMElements.S,
    Samp = DOMElements.Samp,
    Script = DOMElements.Script,
    Section = DOMElements.Section,
    Select = DOMElements.Select,
    Slot = DOMElements.Slot,
    Small = DOMElements.Small,
    Source = DOMElements.Source,
    Span = DOMElements.Span,
    Strong = DOMElements.Strong,
    Style = DOMElements.Style,
    Sub = DOMElements.Sub,
    Summary = DOMElements.Summary,
    Sup = DOMElements.Sup,
    Svg = DOMElements.Svg,
    Table = DOMElements.Table,
    Tbody = DOMElements.Tbody,
    Td = DOMElements.Td,
    Template = DOMElements.Template,
    Textarea = DOMElements.Textarea,
    Tfoot = DOMElements.Tfoot,
    Th = DOMElements.Th,
    Thead = DOMElements.Thead,
    Time = DOMElements.Time,
    Title = DOMElements.Title,
    Tr = DOMElements.Tr,
    Track = DOMElements.Track,
    U = DOMElements.U,
    Ul = DOMElements.Ul,
    Var = DOMElements.Var,
    Video = DOMElements.Video,
    wbr = DOMElements.wbr;

export default withTemplating;
export { A, Abbr, Address, Area, Article, Aside, Audio, B, Base, Bdi, Bdo, Blockquote, Body, Br, Button, Canvas, Caption, Cite, Code, Col, Colgroup, Data, Datalist, Dd, Del, Details, Dfn, Dialog, Div, Dl, Dt, Em, Embed, Fieldset, Figcaption, Figure, Footer, Form, H1, H2, H3, H4, H5, H6, Head, Header, Hgroup, Hr, Html, I, Iframe, Img, Input, Ins, Kbd, Keygen, Label, Legend, Li, Link, Main, Map, Mark, Math, Menu, Menuitem, Meta, Meter, Nav, Noscript, Object$1 as Object, Ol, Optgroup, Option, Output, P, Param, Picture, Pre, Progress, Q, Rb, Rp, Rt, Rtc, Ruby, S, Samp, Script, Section, Select, Slot, Small, Source, Span, Strong, Style, Sub, Summary, Sup, Svg, Table, Tbody, Td, Template, Textarea, Tfoot, Th, Thead, Time, Title, Tr, Track, U, Ul, Var, Video, wbr, Fragment$1 as Fragment };
//# sourceMappingURL=index.es.js.map
